@@ -6,19 +6,19 @@ extern crate diesel;
 #[macro_use]
 extern crate log;
 
-mod schema;
-
 pub mod executor;
-pub mod postgres;
+pub mod queue;
 pub mod scheduler;
+pub mod schema;
 pub mod worker_pool;
 
 pub use executor::*;
-pub use postgres::*;
+pub use queue::*;
 pub use scheduler::*;
+pub use schema::*;
 pub use worker_pool::*;
 
 #[doc(hidden)]
-pub use serde::{Deserialize, Serialize};
+pub use diesel::pg::PgConnection;
 #[doc(hidden)]
 pub use typetag;
